#include <Basis/Screen.hpp>
#include <Graphics/Graphics.hpp>

#include <vector>

using std::vector;

const float Tick = 0.01f;

class TexFb {
public:
	TexFb(unsigned int width, unsigned int height);
	GLuint tex;
	GLuint fb;
	unsigned int width, height;
	void swap(TexFb &other) {
		std::swap(tex, other.tex);
		std::swap(fb, other.fb);
	}
};

class Guidance {
public:
	Guidance(unsigned int width, unsigned int height, vector< Vector3f > const &motion);
	unsigned int width;
	unsigned int height;
	GLuint tex; //TEXTURE_RECTANGLE!
};

const float MaxLifetime = 10.0f;
const float FadeIn = 1.0f;
const float FadeOut = 1.0f;

class Instance {
public:
	unsigned int source;
	Vector2f pos;
	float rot;
	float scale;
	float lifetime;
};

const unsigned int FieldWidth = 256;
const unsigned int FieldHeight = 128;

const unsigned int ParticlesWidth = 256;
const unsigned int ParticlesHeight = 128;

const unsigned int SpawnSize = 32;

class Player : public Screen {
public:
	Player();
	virtual ~Player();
	virtual void handle_event(SDL_Event const &event);
	virtual void update(float elapsed_time);
	virtual void draw();
	void tick();
	void init(); //set to some initial state.

	//Ok we have textures for particle position, velocity, and color:
	GLuint posvel_tex, color_tex;
	GLuint posvel_fb, color_fb;
	GLuint posvel_temp_tex, color_temp_tex;
	GLuint posvel_temp_fb, color_temp_fb;
	//we need a texture for random spawn locations:
	GLuint spawn_tex;
	//For rendering particles, have vertex buffers:
	GLuint posvel_vbo, color_vbo;
	//Field is where we dump velocities:
	unsigned int field_width, field_height;
	vector< TexFb > field;
	vector< TexFb > field_temp;
	TexFb field_color;
	//world_size is aspect ratio of field, but fits (centered) in window.
	Vector2f world_size;
	
	float time_acc;
	unsigned int spawn_point;
	
	//update camera frame, motion?
	unsigned int cam_width;
	unsigned int cam_height;
	GLuint cam_image;
	unsigned int cam_color_width;
	unsigned int cam_color_height;
	float cam_weight;
	GLuint cam_color;
	void new_frame();

	//report a rough score of how well a given instance fits:
	void check_instance(Vector2f pos, float rot, float scale_width, unsigned int width, unsigned int height, GLuint tex, float &sum, Vector2f &vel, float &rot_vel);

	unsigned int sum_width, sum_height;

	GLuint sum_tex, sum_temp_tex;
	GLuint sum_fb, sum_temp_fb;

	vector< Guidance > guidance;
	vector< Instance > instances;

	//For testing, mainly:
	Vector2f mouse_pos;
	float mouse_rot;
	float mouse_scale;

	bool show_boxes;
};
