#include <Basis/Screen.hpp>
#include "Player.hpp"

#include <Graphics/Graphics.hpp>

#include <SDL.h>

#include <Basis/Error.hpp>

#include <string>
#include <iostream>

#include <cv.h>
#include <highgui.h>

using std::string;
using std::cout;
using std::cerr;
using std::endl;

CvCapture *capture = NULL;

int main(int argc, char **argv) {
	if (argc == 1 || (argc == 2 && string(argv[1]).size() == 1)) {
		capture = cvCaptureFromCAM( argc == 2 ? argv[1][0] - '0' : 0 );
	} else if (argc == 2) {
		capture = cvCaptureFromAVI( argv[1] ); 
	}
	if (!capture) {
		cerr << "Could not initialize capturing..." << endl;
		return -1;
	}


	if (SDL_Init(SDL_INIT_TIMER) != 0) {
		LOG_ERROR("Could not initialize sdl: " << SDL_GetError());
		return 1;
	}

	Graphics::screen_x = 1024;
	Graphics::screen_y = 768;
	Graphics::fullscreen = true;

	if (!Graphics::init(Graphics::NEED_STENCIL)) {
		LOG_ERROR("Could not initialize graphics, not continuing.");
		exit(1);
	}

	//Some quick basic GL state:
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glScalef(1.0f / Graphics::aspectf(), 1.0f, 1.0f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_LINE_SMOOTH);
	glEnable(GL_POINT_SMOOTH);

	//we'll draw our own cursor, thank you.
	//SDL_ShowCursor(SDL_DISABLE);
	//if (argc > 2) {
	//	SDL_WM_GrabInput(SDL_GRAB_ON);
	//}
	
	SDL_EnableUNICODE(1);

	srand(time(0));

	Player player;

	start_screen(&player);

	Graphics::deinit();

	SDL_Quit();
}
